import {Component, forwardRef, Input, OnChanges, SimpleChanges, ViewChild} from '@angular/core';
import {
  AbstractControl,
  ControlValueAccessor,
  NG_VALIDATORS,
  NG_VALUE_ACCESSOR,
  ValidationErrors,
  Validator
} from '@angular/forms';
import {SaveRegularObservationsRequestBloodPressure} from '@nspop/gm-web-facade-api';
import {BloodPressureInputComponent} from '../blood-pressure-input/blood-pressure-input.component';

export const APP_BLOOD_PRESSURE_FORM_FIELD_VALIDATOR: any = {
  provide: NG_VALIDATORS,
  useExisting: forwardRef(() => BloodPressureFormFieldComponent),
  multi: true
};

export const APP_BLOOD_PRESSURE_FORM_FIELD_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => BloodPressureFormFieldComponent),
  multi: true
};

@Component({
  selector: 'app-blood-pressure-form-field',
  templateUrl: './blood-pressure-form-field.component.html',
  styleUrls: ['./blood-pressure-form-field.component.scss'],
  providers: [
    APP_BLOOD_PRESSURE_FORM_FIELD_CONTROL_VALUE_ACCESSOR,
    APP_BLOOD_PRESSURE_FORM_FIELD_VALIDATOR
  ],
})
export class BloodPressureFormFieldComponent implements ControlValueAccessor, OnChanges, Validator {

  @ViewChild('input') bloodPressureInput: BloodPressureInputComponent | undefined;

  constructor() { }
  @Input() label: string = 'Blodtryk';
  @Input() name = '';
  @Input() unit = 'mm/Hg';
  @Input() notRelevant: boolean | undefined;

  public disabled = false;
  public required = false;
  public invalid = false;

  innerModel: SaveRegularObservationsRequestBloodPressure | undefined;
  // When notRelevant is toggled on, we store the old value, so we can re-fill it it gets turned off again
  private oldValue: SaveRegularObservationsRequestBloodPressure | undefined;
  private dirtyAble = false;
  private onChangeCallback: (newValue: SaveRegularObservationsRequestBloodPressure | undefined) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  get value(): SaveRegularObservationsRequestBloodPressure | undefined {
    return this.innerModel;
  }

  set value(value: SaveRegularObservationsRequestBloodPressure | undefined) {
    if (value !== this.innerModel) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value);
      }
    }
  }

  // Set touched on blur
  onBlur(): void {
    this.onTouchedCallback();
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.disabled = isDisabled;
  }

  writeValue(newValue: any): void {
    this.value = newValue;
  }

  public makeDirtyAble(): void {
    this.dirtyAble = true;
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.notRelevant) {
      if (this.notRelevant) {
        this.oldValue = this.innerModel;
        this.writeValue(undefined);
      } else {
        this.writeValue(this.oldValue);
      }
    }
  }

  validate(control: AbstractControl): ValidationErrors | null {
    return this?.bloodPressureInput?.errorState
      ? {onlyOneFieldIsFilled: 'Only one value is filled. For blood pressure both systolic and diastolic should be filled.'}
      : null;
  }
}
