import {Component, Input, OnInit} from '@angular/core';
import {ChartDataset, ChartOptions, Tooltip, TooltipItem, TooltipModel} from 'chart.js';
import {Observable} from 'rxjs';
import {ObservationTable, ObservationTableMotherRowsInner} from '@nspop/gm-web-facade-api';

@Component({
  selector: 'app-line-chart',
  templateUrl: './line-chart.component.html',
  styleUrls: ['./line-chart.component.scss']
})
export class LineChartComponent implements OnInit {
  constructor() { }
  @Input() public data: Observable<ObservationTable | undefined> | undefined;
  public lineChartData: ChartDataset[] = [];
  public lineChartOptions: ChartOptions = {
    borderColor: 'black',
    backgroundColor: '#FFF',
    responsive: true,
    maintainAspectRatio: false,
    layout: {
      padding: {
        left: 50,
        right: 50,
        top: 20,
        bottom: 20
      }
    },
    scales: {
      xAxes: {
        beginAtZero: true,
        suggestedMax: 40,
      },
      yAxes: {
        beginAtZero: true,
        max: 40,
        ticks: { stepSize: 10 }
      } 
    }
  };
  public lineChartLegend = true;
  public lineChartPlugins = [];

  private static getGestationAge(gestationAge: number): string {
    const week = Math.floor(gestationAge);
    const decimal = gestationAge % 1;
    const dayOfWeek = decimal / (1 / 7);
    const gestationDay = dayOfWeek >= 1 ? dayOfWeek - 1 : dayOfWeek;
    return [week, '+', Math.floor(gestationDay)].join('');
  }

  ngOnInit(): void {
    if (this.data) {
      this.data.subscribe( data => {
        if (data?.mother) {
          const motherRows = data.mother.rows;
          if (motherRows) {
            const rowData = this.getMotherRows(motherRows);
            this.lineChartData = [
              {
                label: 'Symfyse-fundusmål',
                data: rowData,
                pointRadius: 6,
                pointBackgroundColor: '#4228C4',
                pointBorderColor: '#FFF',
                borderWidth: 2
              }
            ];
          }
        }
      });
    }
  }

  public getMotherRows(motherRows: ObservationTableMotherRowsInner[]): { x: number | undefined; y: number | undefined }[] {
    return motherRows
      .filter((row)  => row && row.pubismeasurement?.value && row.weeksIntoPregnancy)
      .map(row => {
        return { y: row?.pubismeasurement?.value, x: row.weeksIntoPregnancy };
    });
  }
}
