import {Component, OnInit} from '@angular/core';
import {DataCardService} from '../api/data-card.service';
import {SwitchEncounterComponent} from '../switch-encounter/switch-encounter.component';
import {DataCard} from '@nspop/gm-web-facade-api';
import {MatDialog} from '@angular/material/dialog';
import {PatientContextService} from '../services/patient-context.service';
import {Router} from '@angular/router';
import {appUrls} from '../../utils/URL_STRING_LITERALS';
import {WindowService} from '../services/window.service';
import {CarePlanService} from '../api/care-plan.service';
import {EncounterService} from '../services/encounter.service';

@Component({
  selector: 'app-no-active-pathway',
  templateUrl: './no-active-pathway.component.html',
  styleUrls: ['./no-active-pathway.component.scss']
})
export class NoActivePathwayComponent implements OnInit {

  public isThereAnyEarlierPregnancies: boolean = false;

  constructor(
    private dataCardService: DataCardService,
    private dialog: MatDialog,
    private patientContextService: PatientContextService,
    private encounterService: EncounterService,
    private router: Router
  ) {
  }

  public ngOnInit(): void {
    this.dataCardService.allDataCards$.subscribe((dataCards) => {
      this.isThereAnyEarlierPregnancies = dataCards.some((dataCard) => dataCard.documentStatus === 'CLOSED');
    });
  }

  public createPathway(): void {
    this.dataCardService.create().subscribe(() => {
    // Trigger new data loading and navigation
      this.dataCardService.refresh().then(() => this.router.navigate([appUrls.pregnancy]));
    });
  }

  public openSwitchEncounterDialog(): void {
    const dialogRef = this.dialog.open(SwitchEncounterComponent);
    dialogRef.afterClosed().subscribe(async (result: DataCard) => {
      if (result) {
        await this.encounterService.switchEncounter(result.episodeOfCareIdentifier);
        await this.router.navigate([appUrls.activities]);
      }
    });
  }
}
