import { ComponentFixture, TestBed } from '@angular/core/testing';

import { ScreeningsComponent } from './screenings.component';
import {DataCard} from '@nspop/gm-web-facade-api';
import {JsonTransformerUtil} from '../../../utils/json-transformer.util';
import {FormsModule, NgForm} from '@angular/forms';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {calculateOffsetDate} from '../../../utils/dates';

describe('ScreeningsComponent', () => {
  let component: ScreeningsComponent;
  let fixture: ComponentFixture<ScreeningsComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ ScreeningsComponent ],
      imports: [FormsModule, MatIconTestingModule],
      providers: [
        {provide: NgForm, useValue: new NgForm([], [])},
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(ScreeningsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('only registers changes if model has structural changes', () => {
    const dataCard1 = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    const dataCard2 = JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(dataCard1))) as any as DataCard;
    // tslint:disable-next-line:no-non-null-assertion It's not null because it's a copy of dataCard1 which has the property set...
    dataCard2.previousAbortionsSection!.abortions!.push({
      year: undefined,
      type: 'induced',
      gestationLength: {
        value: 6,
        unit: 'w'
      },
      indication: 'whatever'
    });
    component.registerOnTouched(() => 'meep meep');
    const onChange = jasmine.createSpy('onChange');
    component.writeValue(dataCard1 );
    component.registerOnChange(onChange);
    component.writeValue(JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(dataCard1))) as any as DataCard);
    expect(onChange).toHaveBeenCalledTimes(0);
    component.writeValue(dataCard2 as any as DataCard);
    expect(onChange).toHaveBeenCalledTimes(1);
  });
  it('should set all dates when they are all undefined', () => {
    const newDate = new Date(2020, 6, 20);
    component.writeValue({} as DataCard);
    expect(component.value?.hepatitisBDate).toBeUndefined();
    expect(component.value?.hemoglobinopathyDate).toBeUndefined();
    component.hepatitisDateChanged(newDate);
    expect(component.value?.hepatitisBDate?.getDate()).toBe(calculateOffsetDate(newDate)?.getDate());
    expect(component.value?.hemoglobinopathyDate?.getDate()).toBe(calculateOffsetDate(newDate)?.getDate());
  });
  it('should not set all dates some are already defined', () => {
    const originalDate = new Date(2020, 6, 20);
    component.writeValue({} as DataCard);
    component.hepatitisDateChanged(originalDate);
    expect(component.value?.hepatitisBDate?.getDate()).toBe(calculateOffsetDate(originalDate)?.getDate());
    expect(component.value?.hemoglobinopathyDate?.getDate()).toBe(calculateOffsetDate(originalDate)?.getDate());
    const newDate = new Date(2022, 10, 13);
    component.hepatitisDateChanged(newDate);
    expect(component.value?.hemoglobinopathyDate?.getDate()).toBe(calculateOffsetDate(originalDate)?.getDate());
    expect(component.value?.hepatitisBDate?.getDate()).toBe(calculateOffsetDate(newDate)?.getDate());
  });

  it('should not write anything if value is undefined', () => {
    const originalDate = new Date(2020, 6, 20);
    component.hepatitisDateChanged(originalDate);
    expect(component.value).toBeUndefined();
  });
});
